// BruchAddiererView.java                           MM 2015
// MVC-View

import java.util.*;                             // Observer
import javax.swing.*;               // JFrame, JButton, ...

/**
 * View zum BruchAddiererModel.
 */
public class BruchAddiererView
  extends JFrame
  implements Observer
{
/* ----------------------------------------------------- */
                                                     // GUI
/**
 * Konstante, Startzustand, alle Werte geloescht.
 */
  public static final String ACTION_C = " C ";

/**
 * Konstante, Addition.
 */
  public static final String ACTION_ADD = " + ";

/* ----------------------------------------------------- */
                                                     // GUI
/**
 * Textfeld fuer Zaehler.
 */
  private JTextField tfZaehler;

/**
 * Textfeld fuer Nenner.
 */
  private JTextField tfNenner;

/**
 * Button fuer C.
 */
  private JButton btC;

/**
 * Button fuer +.
 */
  private JButton btAdd;

/**
 * Ergebnislabel.
 */
  private JLabel lbResult;

/* ----------------------------------------------------- */
                                                     // MVC
/**
 * Mathematisches Model,
 * enthaelt Funktionalitaet des Bruchaddieres.
 */
  private BruchAddiererModel model;

/**
 * Zum View gehoeriger Cotroller.
 */
  private BruchAddiererController controller;

/* ----------------------------------------------------- */
                                        // MVC-Installation
/**
 * Konstruktor, setzt Ueberschrift und installiert MVC.
 * @param model Model, welches dargestellt werden soll
 * @param titel Ueberschrift
 */
  public BruchAddiererView
  ( BruchAddiererModel model, String titel)
  {
// Titelzeile
    super( titel);
// Model
    this.model = model;
    this.model.addObserver( this);
// Controller
    controller = makeController();
// View
    makeView();
  }

/**
 * Erzeugen des Controllers,
 * Empfaenger fuer Ereignisse.
 * die Kontrolle der Eignisssteuerung.
 * @return Controller fuer View
 */
  private BruchAddiererController makeController()
  {
    return new BruchAddiererController( model, this);
  }

/**
 * Erzeugt View,
 * baut die grafische Oberflaeche auf,
 * Controller verarbeitet Fensterereignisse.
 */
  private void makeView()
  {
// Container
    getContentPane().add( makeBruchAddiererBox());

// Fenster mit Controller als Listener
    addWindowListener( controller);
    pack();
    setVisible( true);
  }

/* ----------------------------------------------------- */
                                                   // Boxen
/**
 * Senkrechte Box fuer Ein-, Ausgaben und Button
 */
  private Box makeBruchAddiererBox()
  {
    Box box = Box.createVerticalBox();
    box.add( Box.createVerticalStrut(20));
    box.add( makeBoxZaehler());
    box.add( Box.createVerticalStrut(20));
    box.add( makeBoxNenner());
    box.add( Box.createVerticalStrut(20));
    box.add( makeBoxBt());
    box.add( Box.createVerticalStrut(20));
    box.add( makeBoxErg());

    return box;
  }

/**
 * Waagerechte Box fuer Zaehlereingabe.
 */
  private Box makeBoxZaehler()
  {
    JLabel lbZaehler = new JLabel( "Zaehler:");
    tfZaehler = new JTextField( 5);
    tfZaehler.setHorizontalAlignment( JTextField.RIGHT);
    tfZaehler.addKeyListener( controller);      // Eingaben

    Box box = Box.createHorizontalBox();
    box.add( lbZaehler);
    box.add( Box.createHorizontalStrut(20));
    box.add( tfZaehler);

//Startwert
    tfZaehler.setText( "");
// Fokus
    tfZaehler.requestFocus();

    return box;
  }

/**
 * Waagerechte Box fuer Nennereingabe.
 */
  private Box makeBoxNenner()
  {
    JLabel lbNenner = new JLabel( "Nenner:");
    tfNenner = new JTextField( 5);
    tfNenner.setHorizontalAlignment( JTextField.RIGHT);
    tfNenner.addKeyListener( controller);       // Eingaben

    Box box = Box.createHorizontalBox();
    box.add( lbNenner);
    box.add( Box.createHorizontalStrut(20));
    box.add( tfNenner);

//Startwert
    tfNenner.setText( "");

    return box;
  }

/**
 * Waagerechte Box fuer Steuerbutton.
 */
  private Box makeBoxBt()
  {
    btC = new JButton( ACTION_C);
    btC.addActionListener( controller);

    btAdd = new JButton( ACTION_ADD);
    btAdd.addActionListener( controller);

    Box box = Box.createHorizontalBox();
    box.add( btC);
    box.add( btAdd);

    return box;
  }

/**
 * Waagerechte Box fuer Ergebnisausgabe.
 */
  private Box makeBoxErg()
  {
    JLabel lbErgebnis = new JLabel( "Ergebnis:");
    lbResult = new JLabel( model.toString());

    Box box = Box.createHorizontalBox();
    box.add( lbErgebnis);
    box.add( Box.createHorizontalStrut(20));
    box.add( lbResult);
    return box;
  }

/* ----------------------------------------------------- */
                        // Dienstleistungen fuer Controller
                                              // Fokusieren
/**
 * Fokus auf Zaehlerein-/ausgabe.
 */
  public void setFocusZaehler()
  {
    tfZaehler.requestFocus();
  }

/**
 * Fokus auf Nennerein-/ausgabe.
 */
  public void setFocusNenner()
  {
    tfNenner.requestFocus();
  }

/**
 * Fokus auf Additionsbutton.
 */
  public void setFocusAdd()
  {
    btAdd.requestFocus();
  }

                         // Textfelder als Ereignisquellen
/**
 * Ereignisquelle ist Zaehlertextfeld
 * @param quelle Ereignisquelle
 */
  public boolean istZaehler( Object quelle)
  {
    return quelle == tfZaehler;
  }

/**
 * Ereignisquelle ist Nennertextfeld.
 * @param quelle Ereignisquelle
 */
  public boolean istNenner( Object quelle)
  {
    return quelle == tfNenner;
  }

/**
 * Lesen der Zahl aus Textfeld.
 * @param quelle Ereignisquelle
 */
  public String getText( Object quelle)
  {
    return ((JTextField)quelle).getText();
  }

/**
 * Zuruecksetzen des Zaehlers im Textfeld.
 */
  public void resetZaehler()
  {
     tfZaehler.setText( "");
 }

/**
 * Zuruecksetzen des Nenners im Textfeld.
 */
  public void resetNenner()
  {
    tfNenner.setText( "");
  }

/* ----------------------------------------------------- */
/**
 * Abfangen von Eingabefehlern
 */
  public void setFehlerDialog( String text)
  {
    JOptionPane.showMessageDialog( this, text,
    "Fehlermeldung", JOptionPane.ERROR_MESSAGE);
  }

/* ----------------------------------------------------- */
                                      // MVC-Deinstallation
/**
 * Entfernt Fenster, deinstalliert MVC.
 */
  public void release()
  {
// View (Fenster)
    dispose();
// Controller
    controller.release();
    controller = null;
// Model
    model.deleteObserver( this);
    model = null;
  }

/* ------------------------------------------------------*/
                                        // Observer-Methode
/**
 * Ueberschreibt Interfacemethode,
 * legt Reaktion auf Aenderungen fest.
 * @param m Model, welches Aenderungen meldet
 * @param o geaenderte Objekte
 */
  public void update( Observable m, Object o)
  {
    if( model == m)
    {
      tfZaehler.setText( "");
      tfNenner.setText( "");
      lbResult.setText( "" + model);
    }
  }
}